<?php
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// P A R S E R D F
	// version 2.1.0
	//
	// Loads an parses an RSS/RDF/Atom file (usually used for headline syndication or news feeds). Provides a simply
	// interface to the data contained within the file. This can be used to generate lists/tables/pages/etc using
	// said headlines.
	//
	// Vrai Stacey <vrai@acherondevelopment.com>
	//
	// $Id: parse_rdf.php 1618 2009-06-11 19:18:06Z vrai $
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Globals - required to circumvent PHP 4's terrible OO implementation

	$RDFFile_cache = new RDFFile_Cache ( );
	$RDFFile_current = false;

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Access Class

	class RDFFile
	{
		function RDFFile ( $filename )
		{
			global $RDFFile_cache;
			global $RDFFile_current;

			// Set current working filename and initialise the cache entry
			$this->filename = $filename;
			$RDFFile_current = $filename;
			$RDFFile_cache->initialise ( );

			// When was the file last modified?
			$this->lastmodtime = filemtime ( $this->filename );

			// Open the file handle
			if ( ! ( $handle = fopen ( $filename, 'r' ) ) )
				die ( "Could not open XML file $filename" );

            // Read in the first line and attempt to extract the file encoding, then move to the file pointer back to the start of the file
            $encoding = RDFFile_getEncodingFromLine ( fgets ( $handle, 1024 ), "ISO-8859-1" );
            if ( fseek ( $handle, 0, SEEK_SET ) < 0 )
                die ( "Failed to moved file pointer back to start of $filename" );

			// Setup the XML parser
			$parser = xml_parser_create ( $encoding );
			xml_set_element_handler ( $parser, "RDFFile_startElement", "RDFFile_endElement" );
			xml_set_character_data_handler ( $parser, 'RDFFile_textElement' );

			// Read in the file and parse it
			while ( $data = fread ( $handle, 16384 ) )
			if ( ! xml_parse ( $parser, $data, feof ( $handle ) ) )
				die ( sprintf ( "XML error: %s at line %d", xml_error_string ( xml_get_error_code ( $parser ) ),
					                                        xml_get_current_line_number ( $parser ) ) );

			// Clear the current state
			$RDFFile_current = false;
		}

		// Returns   : Array of channel field names
		function getFieldnames ( )
		{
			global $RDFFile_cache;
			return $RDFFile_cache->getFieldnames ( $this->filename );
		}
		
		// Arguments :
		//   - Name of channel field
		// Returns   : Value of said channel field
		function getField ( $field )
		{
			global $RDFFile_cache;
			return $RDFFile_cache->getField ( $this->filename, $field );
		}

		// Arguments :
		//   - Array of channel field names
		// Returns   : Array of channel field values, in same order as provided names
		function getFields ( $fields )
		{
			global $RDFFile_cache;
			$values = array ( );
			foreach ( $fields as $field )
				array_push ( $values, $RDFFile_cache->getField ( $this->filename, $field ) );
			return $values;
		}

		// Returns   : Number of items in files
		function getNumberOfItems ( )
		{
			global $RDFFile_cache;
			return $RDFFile_cache->getNumberOfItems ( $this->filename );
		}

		// Arguments :
		//   - Index of item
		// Returns   : Array of field names of item at provided index
		function getItemFieldnames ( $index )
		{
			global $RDFFile_cache;
			return $RDFFile_cache->getItemFieldnames ( $this->filename, $index );
		}

		// Arguments :
		//   - Index of item
		//   - Name of item field
		// Returns   : Value of said field for item at provided index
		function getItemField ( $index, $field )
		{
			global $RDFFile_cache;
			return $RDFFile_cache->getItemField ( $this->filename, $index, $field );
		}

		// Arguments :
		//   - Index of item
		//   - Array of item field names
		// Returns   : Array of item field values, for item at provided index, in same order as provided names
		function getItemFields ( $index, $fields )
		{
			global $RDFFile_cache;
			$values = array ( );
			foreach ( $fields as $field )
				array_push ( $values, $RDFFile_cache->getItemField ( $this->filename, $index, $field ) );
			return $values;
		}

		// Returns   : Last modified time of the XML file, in seconds since the epoch
		function getFileModifiedTime ( )
		{
			return $this->lastmodtime;
		}
	}
	

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Cache classes

	class RDFFile_File
	{
		function RDFFile_File ( $filename )
		{
			// Information members
			$this->name = $filename;
			$this->details = array ( );
			$this->items = array ( );

			// State members - used during XML parsing
			$this->state = array ( 'root' );
			$this->buffer = '';
		}
	}

	class RDFFile_Item
	{
		function RDFFile_Item ( )
		{
			$this->details = array ( );
		}

		function setDetails ( $key, $value )
		{
			$this->details [ $key ] = $value;
		}

		function getFieldnames ( )
		{
			return array_keys ( $this->details );
		}

		function getField ( $field )
		{
			return $this->details [ $field ];
		}
	}

	class RDFFile_Cache
	{
		function RDFFile_Cache ( )
		{
			$this->elements = array ( );
		}
		
		//////////////////////////////////////////////////////////////////////
		// Access methods
		
		function getFieldnames ( $filename )
		{
			return array_keys ( $this->elements [ $filename ]->details );
		}
		
		function getField ( $filename, $field )
		{
			return $this->elements [ $filename ]->details [ $field ];
		}

		function getNumberOfItems ( $filename )
		{
			return sizeof ( $this->elements [ $filename ]->items );
		}

		function getItemFieldnames ( $filename, $index )
		{
			return $this->elements [ $filename ]->items [ $index ]->getFieldnames ( );
		}

		function getItemField ( $filename, $index, $field )
		{
			return $this->elements [ $filename ]->items [ $index ]->getField ( $field );
		}

		//////////////////////////////////////////////////////////////////////
		// File methods

		function initialise ( )
		{
			global $RDFFile_current;
			$this->elements [ $RDFFile_current ] = new RDFFile_File ( $RDFFile_current );
		}

		function setDetails ( $key, $value )
		{
			global $RDFFile_current;
			$this->elements [ $RDFFile_current ]->details [ $key ] = $value;
		}

		//////////////////////////////////////////////////////////////////////
		// Item methods

		function addItem ( )
		{
			global $RDFFile_current;
			array_push ( $this->elements [ $RDFFile_current ]->items, new RDFFile_Item ( ) );
		}

		function setItemDetails ( $key, $value )
		{
			global $RDFFile_current;
			$this->elements [ $RDFFile_current ]->items [ sizeof ( $this->elements [ $RDFFile_current ]->items ) - 1 ]->setDetails ( $key, $value );
		}

		//////////////////////////////////////////////////////////////////////
		// State methods

		function inState ( $state )
		{
			global $RDFFile_current;
			return strcmp ( $this->getState ( ), $state ) == 0;
		}

		function getState ( )
		{
			global $RDFFile_current;
			return $this->elements [ $RDFFile_current ]->state [ 0 ];
		}

		function pushState ( $state )
		{
			global $RDFFile_current;
			array_unshift ( $this->elements [ $RDFFile_current ]->state, $state );
		}

		function popState ( )
		{
			global $RDFFile_current;
			return array_shift ( $this->elements [ $RDFFile_current ]->state );
		}

		//////////////////////////////////////////////////////////////////////
		// Buffer methods

		function clearBuffer ( )
		{
			global $RDFFile_current;
			$this->elements [ $RDFFile_current ]->buffer = '';
		}

		function getBuffer ( )
		{
			global $RDFFile_current;
			return $this->elements [ $RDFFile_current ]->buffer;
		}

		//////////////////////////////////////////////////////////////////////
		// Tag handling methods

		function openTag ( $name, $attrs )
		{
			switch ( $name )
			{
				case 'channel':
					if ( $this->inState ( 'root' ) )
						$this->pushState ( 'channel' );
					break;

                case 'feed':
                    if ( $this->inState ( 'root' ) )
                        $this->pushState ( 'feed' );
                    break;

				case 'item':
                case 'entry':
					$this->pushState ( $name );
					$this->addItem ( );
					break;

                case 'link':
                    if ( $attrs [ 'REL' ] != 'self' && $attrs [ 'HREF' ] )
                        switch ( $this->getState ( ) )
                        {
                            case 'feed':
                                $this->setDetails ( $name, $attrs [ 'HREF' ] );
                                break;

                            case 'entry':
							    $this->setItemDetails ( $name, $attrs [ 'HREF' ] );
                                break;
                        }
                    break;
			};
		}
		
		function closeTag ( $name, $attrs )
		{
            $isAtomTag = $this->inState ( 'feed' ) ||
                         $this->inState ( 'entry' );

			switch ( $name )
			{
				case 'channel':
					if ( $this->inState ( 'channel' ) )
						$this->popState ( );
					break;

                case 'feed':
                    if ( $this->inState ( 'feed' ) )
                        $this->popState ( );
                    break;

				case 'item':
                case 'entry':
					$this->popState ( );
					break;

				case 'link':
                    if ( $isAtomTag )
                        break;
                    // If not in an Atom tag, FALL THROUGH
                case 'content':
                    if ( $isAtomTag )
                        $name = 'description';
                    // Re-names "content" tag to "description" for Atom tags, then FALL THROUGH
				case 'title':
				case 'description':
				case 'copyright':
					switch ( $this->getState ( ) )
					{
						case 'channel':
                        case 'feed':
							$this->setDetails ( $name, $this->getBuffer ( ) );
							break;

						case 'item':
                        case 'entry':
							$this->setItemDetails ( $name, $this->getBuffer ( ) );
							break;
					}
					break;
			};
			
			$this->clearBuffer ( );
		}

		function textTag ( $text )
		{
			global $RDFFile_current;
			
			if ( $text )
				$this->elements [ $RDFFile_current ]->buffer .= $text;
		}
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// XML Parser Functions (and global)

	function RDFFile_startElement ( $parser, $name, $attrs )
	{
		global $RDFFile_cache;

		$name = strtolower ( trim ( $name ) ); 

		$RDFFile_cache->openTag ( $name, $attrs );
	}

	function RDFFile_endElement ( $parser, $name )
	{
		global $RDFFile_cache;
		
		$name = strtolower ( trim ( $name ) );

		$RDFFile_cache->closeTag ( $name, $attrs );
	}

	function RDFFile_textElement ( $parser, $text )
	{
		global $RDFFile_cache;

		$text = $text;

		$RDFFile_cache->textTag ( $text );
	}

    function RDFFile_getEncodingFromLine ( $line, $default )
    {
        $matches = array ( );
        if ( preg_match ( "/<\?xml.*encoding=[\"'](.+)[\"']/i", $line, $matches ) && sizeof ( $matches ) > 1 )
            return $matches [ 1 ];
        return $default;
    }
?>
